@@ -47,7 +47,7 @@ class CompressedValue:
         '''Set uncompressed value.'''
 
         out = BytesIO()
-        gzip.GzipFile(self.name, mode='wb', fileobj=out).write(value)
+        gzip.GzipFile(self.name, mode='wb', fileobj=out, mtime=0).write(value)
         self.gzipvalue = out.getvalue()
         self.legacy_zlib = False
 
@@ -188,6 +188,70 @@ class ProblemReport(UserDict):
 
         self.old_keys = set(self.data.keys())
 
+    def extract_keys(self, file, bin_keys, dir):
+        '''Extract only one binary element from the problem_report
+
+        Binary elements like kernel crash dumps can be very big. This method
+        extracts directly files without loading the report into memory.
+        '''
+        self._assert_bin_mode(file)
+        # support singe key and collection of keys
+        if isinstance(bin_keys, str):
+            bin_keys = [bin_keys]
+        key = None
+        value = None
+        missing_keys = list(bin_keys)
+        b64_block = {}
+        bd = None
+        out = None
+        for line in file:
+            # Identify the bin_keys we're looking for
+            while not line.startswith(b' '):
+                (key, value) = line.split(b':', 1)
+                if not _python2:
+                    key = key.decode('ASCII')
+                if key not in missing_keys:
+                    break
+                b64_block[key] = False
+                missing_keys.remove(key)
+                value = value.strip()
+                if value == b'base64':
+                    value = b''
+                    b64_block[key] = True
+                    try:
+                        bd = None
+                        with open(os.path.join(dir, key), 'wb') as out:
+                            for line in file:
+                                # continuation line
+                                if line.startswith(b' '):
+                                    assert (key is not None and value is not None)
+                                    if b64_block[key]:
+                                        l = base64.b64decode(line)
+                                        if bd:
+                                            out.write(bd.decompress(l))
+                                        else:
+                                            # lazy initialization of bd
+                                            # skip gzip header, if present
+                                            if l.startswith(b'\037\213\010'):
+                                                bd = zlib.decompressobj(-zlib.MAX_WBITS)
+                                                out.write(bd.decompress(self._strip_gzip_header(l)))
+                                            else:
+                                                # legacy zlib-only format used default block
+                                                # size
+                                                bd = zlib.decompressobj()
+                                                out.write(bd.decompress(l))
+                                else:
+                                    break
+                    except IOError:
+                        raise IOError('unable to open %s' % (os.path.join(dir, key)))
+                else:
+                    break
+        if missing_keys:
+            raise KeyError('Cannot find %s in report' % ', '.join(missing_keys))
+        if False in b64_block.values():
+            raise ValueError('%s has no binary content' %
+                             [item for item, element in b64_block.items() if element is False])
+
     def has_removed_fields(self):
         '''Check if the report has any keys which were not loaded.
 
@@ -451,7 +515,7 @@ class ProblemReport(UserDict):
 
         keys = sorted(self.data.keys())
 
-        text = b''
+        text = ''
         attachments = []
 
         if 'ProblemType' in keys:
@@ -488,7 +552,7 @@ class ProblemReport(UserDict):
                     attach_value = f.read()
                 else:
                     io = BytesIO()
-                    gf = gzip.GzipFile(k, mode='wb', fileobj=io)
+                    gf = gzip.GzipFile(k, mode='wb', fileobj=io, mtime=0)
                     while True:
                         block = f.read(1048576)
                         if block:
@@ -528,17 +592,16 @@ class ProblemReport(UserDict):
                     assert isinstance(v, unicode)
                 else:
                     assert isinstance(v, str)
-                v = v.encode('UTF-8')
 
                 lines = len(v.splitlines())
                 if size <= 1000 and lines == 1:
                     v = v.rstrip()
-                    text += k.encode() + b': ' + v + b'\n'
+                    text += k + ': ' + v + '\n'
                 elif size <= 1000 and lines <= attach_treshold:
-                    text += k.encode() + b':\n '
-                    if not v.endswith(b'\n'):
-                        v += b'\n'
-                    text += v.strip().replace(b'\n', b'\n ') + b'\n'
+                    text += k + ':\n '
+                    if not v.endswith('\n'):
+                        v += '\n'
+                    text += v.strip().replace('\n', '\n ') + '\n'
                 else:
                     # too large, separate attachment
                     att = MIMEText(v, _charset='UTF-8')
