@@ -2223,7 +2223,7 @@ No symbol table info available.
             return
 
         (session, timestamp) = ret
-        self.assertIn('/user', session)
+        self.assertNotEqual(session, '')
         # session start must be >= 2014-01-01 and "now"
         self.assertLess(timestamp, time.time())
         self.assertGreater(timestamp,
@@ -2231,7 +2231,7 @@ No symbol table info available.
 
     def test_get_timestamp(self):
         r = apport.Report()
-        self.assertAlmostEqual(r.get_timestamp(), time.time(), delta=1)
+        self.assertAlmostEqual(r.get_timestamp(), time.time(), delta=2)
 
         r['Date'] = 'Thu Jan 9 12:00:00 2014'
         # delta is ±12 hours, as this depends on the timezone that the test is
