@@ -533,6 +533,13 @@ class T(unittest.TestCase):
 
         # run test program as user "mail"
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
+
+        if os.path.isdir('/run/systemd/system'):
+            # FIXME: no core file/apport dump at all under systemd
+            self.do_crash(False, command=myexe, expect_corefile=False, uid=8)
+            self.assertEqual(apport.fileutils.get_all_reports(), [])
+            return
+
         # expect the core file to be owned by root
         self.do_crash(command=myexe, expect_corefile=True, uid=8,
                       expect_corefile_owner=0)
@@ -554,6 +561,14 @@ class T(unittest.TestCase):
 
         # run ping as user "mail"
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
+
+        if os.path.isdir('/run/systemd/system'):
+            # FIXME: no core file/apport dump at all under systemd
+            self.do_crash(False, command='/bin/ping', args=['127.0.0.1'],
+                          uid=8)
+            self.assertEqual(apport.fileutils.get_all_reports(), [])
+            return
+
         # expect the core file to be owned by root
         self.do_crash(command='/bin/ping', args=['127.0.0.1'],
                       expect_corefile=True, uid=8,
@@ -569,6 +584,49 @@ class T(unittest.TestCase):
         # this must not be owned by root as it is a setuid binary
         self.assertEqual(st.st_uid, 0, 'report has correct owner')
 
+    @unittest.skipUnless(os.path.exists('/usr/bin/lxc-usernsexec'), 'this test needs lxc')
+    @unittest.skipUnless(os.path.exists('/bin/busybox'), 'this test needs busybox')
+    @unittest.skipIf(os.access('/etc/shadow', os.R_OK), 'this test needs to be run as user')
+    def test_ns_forward_privilege(self):
+        c = os.path.join(self.workdir, 'c')
+        os.makedirs(os.path.join(c, 'dev'))
+        os.mkdir(os.path.join(c, 'mnt'))
+        os.makedirs(os.path.join(c, 'usr/share/apport'))
+        shutil.copy('/bin/busybox', c)
+        with open(os.path.join(c, 'usr/share/apport/apport'), 'w') as f:
+            f.write('''#!/busybox sh
+set -x
+exec 2>/apport.trace
+cat /mnt/1/root/etc/shadow > /mnt/1/root/tmp/pwned
+chmod 644 /mnt/1/root/tmp/pwned
+''')
+            os.fchmod(f.fileno(), 0o755)
+
+        ns_apport = subprocess.Popen(
+            ['lxc-usernsexec', '-m', 'u:0:%i:1' % os.getuid(),
+             '-m', 'g:0:%i:1' % os.getgid(), '--',
+             'lxc-unshare', '-s', 'MOUNT|PID|NETWORK|UTSNAME|IPC', '--', '/bin/sh'],
+            stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT)
+        ns_apport.stdin.write(('''set -x
+cd %s
+mount -o bind . .
+cd .
+mount --rbind /proc mnt
+touch dev/null
+pivot_root . .
+./busybox ls -lh /
+./busybox sh -c 'kill -SEGV $$'
+./busybox sleep 5
+if [ -e /apport.trace ]; then
+    echo "=== apport trace ===="
+    ./busybox cat /apport.trace
+fi
+''' % c).encode())
+        out = ns_apport.communicate()[0].decode()
+        self.assertEqual(ns_apport.returncode, 0, out)
+        self.assertFalse(os.path.exists('/tmp/pwned'), out)
+
     #
     # Helper methods
     #
@@ -587,6 +645,8 @@ class T(unittest.TestCase):
         if pid == 0:
             if uid is not None:
                 os.setuid(uid)
+            # set UTF-8 environment variable, to check proper parsing in apport
+            os.putenv('utf8trap', b'\xc3\xa0\xc3\xa4')
             os.dup2(os.open('/dev/null', os.O_WRONLY), sys.stdout.fileno())
             sys.stdin.close()
             os.setsid()
